#include <TinyWire.h>
#include <twi.h>

//#include <TinyWireS.h>
//#include <usiTwiSlave.h>

#include <SoftwareSerial.h>

#include <FastLED.h>

#define RX 3
#define TX 0
//#define RX 0
//#define TX 3

SoftwareSerial serial(RX,TX);

//#define DOT_STAR_DATA_PIN    PB0
//#define DOT_STAR_CLOCK_PIN   PB1
#define DOT_STAR_DATA_PIN  10
#define DOT_STAR_CLOCK_PIN  9
#define DOT_STAR_NUM_PIXELS_IN_STRIP  30

//#define DOT_STAR_DATA_PIN_ROUND    8
//#define DOT_STAR_CLOCK_PIN_ROUND   7
//#define DOT_STAR_NUM_PIXELS_IN_STRIP_ROUND  20

CRGBArray<DOT_STAR_NUM_PIXELS_IN_STRIP> led_strip;
//CRGB round_strip[DOT_STAR_NUM_PIXELS_IN_STRIP_ROUND];

#define HEARTBEAT_INPUT_PIN 1
#define I2C_SLAVE_ADDRESS   42

void setup() {

  //interanl 8Mhz clock
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

  delay(2000);
  serial.begin(115200);
  serial.print("-- start --");
  serial.flush();

  pinMode(HEARTBEAT_INPUT_PIN, INPUT);
  digitalWrite(HEARTBEAT_INPUT_PIN, HIGH);

  FastLED.addLeds<DOTSTAR, DOT_STAR_DATA_PIN, DOT_STAR_CLOCK_PIN, BGR>(led_strip, DOT_STAR_NUM_PIXELS_IN_STRIP);

  TinyWire.begin(I2C_SLAVE_ADDRESS);
//  Wire.onReceive(wireReceiveEvent); // register interrupt event

  //  FastLED.addLeds<DOTSTAR, DOT_STAR_DATA_PIN_ROUND, DOT_STAR_CLOCK_PIN_ROUND, BGR>(round_strip, DOT_STAR_NUM_PIXELS_IN_STRIP_ROUND);
}

#define NUM_PIXELS_IN_HALF_TRUNK 13
#define NUM_PIXELS_IN_HEART_LINK 5

uint8_t treeLightThroughPatternIndex = 0;
void treeLightThroughPattern(){
  //set next bright pixel
  led_strip[treeLightThroughPatternIndex] = CRGB::Red;

  if (treeLightThroughPatternIndex >= NUM_PIXELS_IN_HEART_LINK) {
    // now, let's copy first half leds to the top half leds 
    led_strip(DOT_STAR_NUM_PIXELS_IN_STRIP-NUM_PIXELS_IN_HALF_TRUNK-1, DOT_STAR_NUM_PIXELS_IN_STRIP-1) = 
       led_strip(NUM_PIXELS_IN_HALF_TRUNK+NUM_PIXELS_IN_HEART_LINK - 1, NUM_PIXELS_IN_HEART_LINK);
  }

  treeLightThroughPatternIndex++;
  if (treeLightThroughPatternIndex >= NUM_PIXELS_IN_HALF_TRUNK + NUM_PIXELS_IN_HEART_LINK){
    treeLightThroughPatternIndex = 0;
  }
  
}

uint8_t heartBeatPatternIndex = 0;
void heartBeatPattern(){
  if (heartBeatPatternIndex < (NUM_PIXELS_IN_HALF_TRUNK + NUM_PIXELS_IN_HEART_LINK)*2){
    treeLightThroughPattern();
  }
  
  led_strip.fadeToBlackBy(120);
  FastLED.delay(20);

  heartBeatPatternIndex++;
  if (heartBeatPatternIndex > (NUM_PIXELS_IN_HALF_TRUNK + NUM_PIXELS_IN_HEART_LINK)*2 + 15) {
     heartBeatPatternIndex = 0;
     treeLightThroughPatternIndex = 0;    
  }
}

int randomPatternFadeIndex = 0;
void randomPattern(){  
  if (randomPatternFadeIndex < 10){
    randomPatternFadeIndex++;
  } else {
    led_strip[random(DOT_STAR_NUM_PIXELS_IN_STRIP)] = CRGB::Red;
    randomPatternFadeIndex = 0;
  }
  led_strip.fadeToBlackBy(10);
  FastLED.delay(20);
}


void addGlitter( fract8 chanceOfGlitter) 
{
  if( random8() < chanceOfGlitter) {
    led_strip[ random16(DOT_STAR_NUM_PIXELS_IN_STRIP) ] += CRGB::White;
  }
}

uint8_t gHue = 0;
void rainbow(){
  fill_rainbow( led_strip, DOT_STAR_NUM_PIXELS_IN_STRIP, gHue++, 7);
  addGlitter(80);
  FastLED.delay(20);
}

uint8_t lastReading = 0;
uint8_t available = 0;
uint8_t unavailableCount = 0;

void loop() {

  available = TinyWire.available();
//  serial.print("a: ");
//  serial.println(available);
  if (available != 0 || unavailableCount < 10) { // if bytes were received

    if(available == 0) {
      unavailableCount++;
    } else {
      unavailableCount = 0;
      lastReading = TinyWire.read();  // receive high byte (overwrites previous reading)
//      serial.print("r: ");
//      serial.println(lastReading);
    }
    
    if (lastReading < 100 ){
      rainbow(); 
    } else {
      heartBeatPattern(); 
    }
  } else {
    randomPattern();
  }

}
